@@ -1,6 +1,6 @@
 package Net::DNS::Question;
 #
-# $Id: Question.pm 659 2007-06-21 14:26:19Z olaf $
+# $Id: Question.pm 836 2009-12-30 09:41:53Z olaf $
 #
 use strict;
 BEGIN { 
@@ -12,7 +12,7 @@ use vars qw($VERSION $AUTOLOAD);
 use Carp;
 use Net::DNS;
 
-$VERSION = (qw$LastChangedRevision: 659 $)[1];
+$VERSION = (qw$LastChangedRevision: 800$)[1];
 
 =head1 NAME
 
@@ -42,51 +42,62 @@ queries in in-addr.arpa and ip6.arpa subdomains.
 =cut
 
 sub new {
-	my $class = shift;
+	my $self   = bless {}, shift;
+	my $qname  = shift;
+	my $qtype  = uc (shift || '');
+	my $qclass = uc (shift || '');
 
-	my $qname = defined ($_ = shift) ? $_ : '';
-	my $qtype = uc shift || 'A';
-	my $qclass = uc shift || 'IN';
+	$qname = '' unless defined $qname;	# || ''; is NOT same!
+	$qname =~ s/\.+$//o;			# strip gratuitous trailing dot
 
-	$qname =~ s/\.+$//o;	# strip gratuitous trailing dot
-
- 	# Check if the caller has the type and class reversed.
- 	# We are not that kind for unknown types.... :-)
-	($qtype, $qclass) = ($qclass, $qtype)
-		if exists $Net::DNS::classesbyname{$qtype}
-		and exists $Net::DNS::typesbyname{$qclass};
-
-	# if argument is an IP address, do appropriate reverse lookup
-	if ( $qname =~ m/\d$|[:\/]/o ) {
-		my $type = $qtype =~ m/^(A|AAAA)$/o ? 'PTR' : $qtype;
-		($qname, $qtype) = ($_, $type) if $_ = dns_addr($qname);
+	# tolerate (possibly unknown) type and class in zone file order
+	unless ( exists $Net::DNS::classesbyname{$qclass} ) {
+		( $qtype, $qclass ) = ( $qclass, $qtype )
+				if exists $Net::DNS::classesbyname{$qtype};
+		( $qtype, $qclass ) = ( $qclass, $qtype ) if $qtype =~ /CLASS/;
+	}
+	unless ( exists $Net::DNS::typesbyname{$qtype} ) {
+		( $qtype, $qclass ) = ( $qclass, $qtype )
+				if exists $Net::DNS::typesbyname{$qclass};
+		( $qtype, $qclass ) = ( $qclass, $qtype ) if $qclass =~ /TYPE/;
 	}
 
-	my $self = {	qname	=> $qname,
-			qtype	=> $qtype,
-			qclass	=> $qclass
-			};
+	# if argument is an IP address, do appropriate reverse lookup
+	my $reverse = _dns_addr($qname) if $qname =~ m/:|\d$/o;
+	if ( $reverse ) {
+		$qname = $reverse;
+		$qtype ||= 'PTR';
+	}
 
-	bless $self, $class;
+	$self->{qname}  = $qname;
+	$self->{qtype}  = ( $qtype || 'A' );
+	$self->{qclass} = ( $qclass || 'IN' );
+
+	return $self;
 }
 
 
-sub dns_addr {
+sub _dns_addr {
 	my $arg = shift;	# name or IP address
 
-	# If arg looks like IP4 address then map to in-addr.arpa space
-	if ( $arg =~ /((^|\d+\.)+\d+)($|\/(\d*))/o ) {
-		my @parse = split /\./, $1;
-		my $last = ($_ = ($4 || @parse<<3)) > 24 ? 3 : ($_-1)>>3;
+	# IP address must contain address characters only
+	return undef if $arg =~ m#[^a-fA-F0-9:./]#o;
+
+	# if arg looks like IPv4 address then map to in-addr.arpa space
+	if ( $arg =~ m#(^|:.*:)((^|\d+\.)+\d+)(/(\d+))?$#o ) {
+		my @parse = split /\./, $2;
+		my $prefx = $5 || @parse<<3;
+		my $last = $prefx > 24 ? 3 : ($prefx-1)>>3;
 		return join '.', reverse( (@parse,(0)x3)[0 .. $last] ), 'in-addr.arpa';
 	}
 
-	# If arg looks like IP6 address then map to ip6.arpa space
-	if ( $arg =~ /^((\w*:)+)(\w*)($|\/(\d*))/o ) {
+	# if arg looks like IPv6 address then map to ip6.arpa space
+	if ( $arg =~ m#^((\w*:)+)(\w*)(/(\d+))?$#o ) {
 		my @parse = split /:/, (reverse "0${1}0${3}"), 9;
-		my @xpand = map{/^$/ ? ('0')x(9-@parse) : $_} @parse;
+		my @xpand = map{/./ ? $_ : ('0')x(9-@parse)} @parse;	# expand ::
+		my $prefx = $5 || @xpand<<4;		# implicit length if unspecified
 		my $hex = pack 'A4'x8, map{$_.'000'} ('0')x(8-@xpand), @xpand;
-		my $len = ($_ = ($5 || @xpand<<4)) > 124 ? 32 : ($_+3)>>2;
+		my $len = $prefx > 124 ? 32 : ($prefx+3)>>2;
 		return join '.', split(//, substr($hex,-$len) ), 'ip6.arpa';
 	}
 
@@ -94,6 +105,41 @@ sub dns_addr {
 }
 
 
+=head2 parse
+
+    ($question, $offset) = Net::DNS::Question->parse(\$data, $offset);
+
+Parses a question section record at the specified location within a DNS packet.
+The first argument is a reference to the packet data.
+The second argument is the offset within the packet where the question record begins.
+
+Returns a Net::DNS::Question object and the offset of the next location in the packet.
+
+Parsing is aborted if the question object cannot be created (e.g., corrupt or insufficient data).
+
+=cut
+
+use constant PACKED_LENGTH => length pack 'n2', (0)x2;
+
+sub parse {
+	my ($class, $data, $offset) = @_;
+
+	my ($qname, $index) = Net::DNS::Packet::dn_expand($data, $offset);
+	die 'Exception: corrupt or incomplete data' unless $index;
+
+	my $next = $index + PACKED_LENGTH;
+	die 'Exception: incomplete data' if length $$data < $next;
+	my ($qtype, $qclass) = unpack("\@$index n2", $$data);
+
+	my $self = {	qname	=> $qname,
+			qtype	=> Net::DNS::typesbyval($qtype),
+			qclass	=> Net::DNS::classesbyval($qclass)
+			};
+
+	bless $self, $class;
+
+	return wantarray ? ($self, $next) : $self;
+}
 
 
 #
@@ -128,24 +174,26 @@ known as C<zclass> and refers to the zone's class.
 
 =cut
 
+sub zname  { &qname;  }
+sub ztype  { &qtype;  }
+sub zclass { &qclass; }
+
+
 sub AUTOLOAD {
 	my $self = shift;
 
 	my $name = $AUTOLOAD;
 	$name =~ s/.*://o;
 
-	Carp::croak "$name: no such method" unless exists $self->{$name};
+	croak "$AUTOLOAD: no such method" unless exists $self->{$name};
 
 	return $self->{$name} unless @_;
 
-	$self->{$name} = shift;
+	my $value = shift;
+	$value =~ s/\.+$//o if defined $value;	# strip gratuitous trailing dot
+	$self->{$name} = $value;
 }
 
-
-sub zname  { &qname;  }
-sub ztype  { &qtype;  }
-sub zclass { &qclass; }
-
 =head2 print
 
     $question->print;
@@ -154,7 +202,7 @@ Prints the question record on the standard output.
 
 =cut
 
-sub print {	print shift->string, "\n"; }
+sub print {	print &string, "\n"; }
 
 =head2 string
 
@@ -188,22 +236,29 @@ sub data {
 
 	my $data = $packet->dn_comp($self->{qname}, $offset);
 
-	$data .= pack 'n', Net::DNS::typesbyname(uc $self->{qtype});
-	$data .= pack 'n', Net::DNS::classesbyname(uc $self->{qclass});
-	
+	$data .= pack('n2',	Net::DNS::typesbyname(uc $self->{qtype}),
+				Net::DNS::classesbyname(uc $self->{qclass})
+				);
 	return $data;
 }
 
+
+1;
+__END__
+
+
 =head1 COPYRIGHT
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
-Portions Copyright (c) 2003,2006-2007 Dick Franks.
+Portions Copyright (c) 2003,2006-2009 Dick Franks.
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
+All rights reserved.
+
+This program is free software; you may redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
@@ -213,4 +268,3 @@ RFC 1035 Section 4.1.2
 
 =cut
 
-1;
