@@ -1,15 +1,21 @@
 package Net::DNS::RR;
 #
-# $Id: RR.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: RR.pm 828 2009-12-23 15:03:29Z olaf $
 #
 use strict;
-use vars qw($VERSION $AUTOLOAD);
 
+BEGIN { 
+    eval { require bytes; }
+} 
+
+
+use vars qw($VERSION $AUTOLOAD %rrsortfunct );
 use Carp;
-use Net::DNS;
+use Net::DNS qw (wire2presentation name2labels stripdot);
 use Net::DNS::RR::Unknown;
 
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+
+$VERSION = (qw$LastChangedRevision: 828 $)[1];
 
 =head1 NAME
 
@@ -32,7 +38,6 @@ any of its methods.  If you call an unknown method, you'll get a nasty
 warning message and C<Net::DNS::RR> will return C<undef> to the caller.
 
 =cut
-#' Stupid Emacs (I Don't even USE emacs!) '
 
 # %RR needs to be available within the scope of the BEGIN block.
 # $RR_REGEX is a global just to be on the safe side.  
@@ -47,10 +52,12 @@ BEGIN {
 		AFSDB
 		CNAME
 		CERT
+		DHCID
 		DNAME
 		EID
 		HINFO
 		ISDN
+		KX
 		LOC
 		MB
 		MG
@@ -73,18 +80,27 @@ BEGIN {
 		TXT
 		X25
 		OPT
+		APL
 		SSHFP
+		HIP
+		SPF
+		IPSECKEY
+
 	);
 
 	#  Only load DNSSEC if available
-	# 
 
-	eval { require Net::DNS::RR::SIG; };
+	eval { 	    
+	    local $SIG{'__DIE__'} = 'DEFAULT';
+	    require Net::DNS::RR::SIG; 
+	};
 
 	unless ($@) {
 		$RR{'SIG'} = 1;
-	
-		eval { require Net::DNS::RR::NXT; };
+		eval { 	    
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::NXT; 
+		};
 		
 		unless ($@) {
 		    $RR{'NXT'}	= 1;
@@ -92,7 +108,10 @@ BEGIN {
 		    die $@;
 		}
 		
-		eval { require Net::DNS::RR::KEY; };
+		eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::KEY; 
+		};
 		
 		unless ($@) {
 		    $RR{'KEY'} = 1;
@@ -100,7 +119,10 @@ BEGIN {
 		    die $@;
 		}
 
-	 	eval { require Net::DNS::RR::DS; };
+	 	eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::DS; 
+		};
 
 	 	unless ($@) {
 		    $RR{'DS'} = 1;
@@ -109,18 +131,28 @@ BEGIN {
 		    die $@;
 		}
 
-	 	eval { require Net::DNS::RR::RRSIG; };
+	 	eval { 
+		    local $SIG{'__DIE__'} = 'DEFAULT';
+		    require Net::DNS::RR::RRSIG; 
+		};
 
 	 	unless ($@) {
 		    $RR{'RRSIG'} = 1;
 		    # If RRSIG is available so should the other DNSSEC types
-		    eval { require Net::DNS::RR::NSEC; };
+		    eval {		    
+			local $SIG{'__DIE__'} = 'DEFAULT';
+			require Net::DNS::RR::NSEC; 
+		    };
 		    unless ($@) {
 		      $RR{'NSEC'} = 1;
 		    } else {
 		    die $@;
 		  }
-		    eval { require Net::DNS::RR::DNSKEY; };
+		    eval { 
+			local $SIG{'__DIE__'} = 'DEFAULT';
+			require Net::DNS::RR::DNSKEY; 
+		    };
+
 		    unless ($@) {
 		      $RR{'DNSKEY'} = 1;
 		    } else {
@@ -128,6 +160,49 @@ BEGIN {
 		    }
 		} 
 
+	 	eval { 
+		  local $SIG{'__DIE__'} = 'DEFAULT';
+		  require Net::DNS::RR::DLV; 
+		};
+
+		unless ($@) {
+		  $RR{'DLV'} =1;
+		} else {
+		  # Die only if we are dealing with a version for which DLV is 
+		  # available 
+		  die $@ if defined ($Net::DNS::SEC::HAS_DLV) ;
+
+		}
+
+	 	eval { 
+		  local $SIG{'__DIE__'} = 'DEFAULT';
+		  require Net::DNS::RR::NSEC3; 
+		};
+
+		unless ($@) {
+		  $RR{'NSEC3'} =1;
+		} else {
+		  # Die only if we are dealing with a version for which NSEC3 is		  # available 
+		  die $@ if defined ($Net::DNS::SEC::HAS_NSEC3);
+		}
+		
+		
+	 	eval { 
+		  local $SIG{'__DIE__'} = 'DEFAULT';
+		  require Net::DNS::RR::NSEC3PARAM; 
+		};
+
+		unless ($@) {
+		  $RR{'NSEC3PARAM'} =1;
+		} else {
+		  # Die only if we are dealing with a version for which NSEC3 is 
+		  # available 
+
+		  die $@ if defined($Net::DNS::SEC::SVNVERSION) &&  $Net::DNS::SEC::SVNVERSION > 619;   # In the code since. (for users of the SVN trunk)
+		}
+
+
+
     }
 }
 
@@ -161,7 +236,7 @@ sub build_regex {
  $a     = Net::DNS::RR->new("foo.example.com. 86400 A 10.1.2.3");
  $mx    = Net::DNS::RR->new("example.com. 7200 MX 10 mailhost.example.com.");
  $cname = Net::DNS::RR->new("www.example.com 300 IN CNAME www1.example.com");
- $txt   = Net::DNS::RR->new("baz.example.com 3600 HS TXT 'text record'");
+ $txt   = Net::DNS::RR->new('baz.example.com 3600 HS TXT "text record"');
 
 Returns a C<Net::DNS::RR> object of the appropriate type and
 initialized from the string passed by the user.  The format of the
@@ -205,18 +280,11 @@ is recommended.
 
 =cut
 
-#' Stupid Emacs
 
 
 sub new {
-	if (@_ == 8 && ref $_[6]) {
-		return new_from_data(@_);
-	}
-	
-	if (@_ == 2 || @_ == 3) {
-		return new_from_string(@_);
-	}
-	
+	return new_from_string(@_) if @_ == 2;
+	return new_from_string(@_) if @_ == 3;
 	return new_from_hash(@_);
 }
 
@@ -225,22 +293,19 @@ sub new_from_data {
 	my $class = shift;
 	my ($name, $rrtype, $rrclass, $ttl, $rdlength, $data, $offset) = @_;
 
-	my $self = {
-		'name'		=> $name,
-		'type'		=> $rrtype,
-		'class'		=> $rrclass,
-		'ttl'		=> $ttl,
-		'rdlength'	=> $rdlength,
-		'rdata'		=> substr($$data, $offset, $rdlength),
-
-	};
-
+	my $self = {	name		=> $name,
+			type		=> $rrtype,
+			class		=> $rrclass,
+			ttl		=> $ttl,
+			rdlength	=> $rdlength,
+			rdata		=> substr($$data, $offset, $rdlength)
+			};
 
 	if ($RR{$rrtype}) {
 		my $subclass = $class->_get_subclass($rrtype);
 		return $subclass->new($self, $data, $offset);
 	} else {
-	    return Net::DNS::RR::Unknown->new($self, $data, $offset);
+		return Net::DNS::RR::Unknown->new($self, $data, $offset);
 	}
 
 }
@@ -249,12 +314,49 @@ sub new_from_string {
 	my ($class, $rrstring, $update_type) = @_;
 
 	build_regex() unless $RR_REGEX;
-	
+
 	# strip out comments
-	$rrstring   =~ s/;.*//g;
+	# Comments start with a semi collon and run till end of line.
+	# However if the semi colon is escaped or inside a character string then we should keep it
+	# see e.g. rt.cpan 49035
+	my $loopdetection=length($rrstring);
+	my $cleanstring;
+	while ($rrstring) {
+
+		if ($rrstring=~s/^([^\\;'"]*)//o){       # Anything not special in this context.
+			$cleanstring.=$1;
+		}
+		if ($rrstring=~s/^(\\.)//o){             # Escaped special character
+			$cleanstring.=$1;
+		}
+		
+		if ($rrstring=~s/^((['"]).*(?<!\\)\2)//o){ # Anything within a matching string block. (non escaped terminator)
+			$cleanstring.=$1;
+
+			# Next: Anything within a non matched string upto
+			# the first encountered comment (but first we
+			# want to make sure we captured all the nested
+			# blocks hence elsif
+		}elsif ($rrstring=~s/^((['"]).*((?<!\\);)?)//o){
+			$cleanstring.=$1;
+		}
+		
+		$rrstring=~s/^(;.*\n)//o;  # comment till newline
+		$rrstring=~s/^(;.*$)//o;   # comment till end of string
+		#print STDERR ".";
+
+		confess "Failed stripping:loop will not terminate. Please report this info: ". $cleanstring ."---". $rrstring."\n" 
+		  if ($loopdetection==length($rrstring));
+		$loopdetection=length($rrstring);
+
+
+	}
+	  
 	
-	($rrstring =~ m/$RR_REGEX/xso) || 
-		confess qq|qInternal Error: "$rrstring" did not match RR pat.\nPlease report this to the author!\n|;
+	
+
+	($cleanstring =~ m/$RR_REGEX/xso) || 
+		confess qq|Internal Error: "$rrstring" did not match RR pat.\nPlease report this to the author!\n|;
 
 	my $name    = $1;
 	my $ttl     = $2 || 0;
@@ -264,14 +366,16 @@ sub new_from_string {
 	my $rrtype  = $4 || '';
 	my $rdata   = $5 || '';
 
-	$rdata =~ s/\s+$// if $rdata;
-	$name  =~ s/\.$//  if $name;
+	$rdata =~ s/\s+$//o if $rdata;
+
+	$name  = stripdot($name)  if $name;
+
 
 
 	# RFC3597 tweaks
 	# This converts to known class and type if specified as TYPE###
-	$rrtype  = Net::DNS::typesbyval(Net::DNS::typesbyname($rrtype))      if $rrtype  =~ m/^TYPE\d+/;
-	$rrclass = Net::DNS::classesbyval(Net::DNS::classesbyname($rrclass)) if $rrclass =~ m/^CLASS\d+/;
+	$rrtype  = Net::DNS::typesbyval(Net::DNS::typesbyname($rrtype))      if $rrtype  =~ m/^TYPE\d+/o;
+	$rrclass = Net::DNS::classesbyval(Net::DNS::classesbyname($rrclass)) if $rrclass =~ m/^CLASS\d+/o;
 
 
 	if (!$rrtype && $rrclass && $rrclass eq 'ANY') {
@@ -304,9 +408,9 @@ sub new_from_string {
 			$rrclass = 'NONE';
 			$rrtype  = 'ANY';
 			$rdata   = '';
-		} elsif ($update_type =~ /^(rr_)?add$/) {
+		} elsif ($update_type =~ /^(rr_)?add$/o) {
 			$ttl = 86400 unless $ttl;
-		} elsif ($update_type =~ /^(rr_)?del(ete)?$/) {
+		} elsif ($update_type =~ /^(rr_)?del(ete)?$/o) {
 			$ttl     = 0;
 			$rrclass = $rdata ? 'NONE' : 'ANY';
 		}
@@ -324,17 +428,15 @@ sub new_from_string {
 		'rdata'    => '',
 	};
 
-
-	if ($RR{$rrtype} && $rdata !~ m/^\s*\\#/ ) {
+	if ($RR{$rrtype} && $rdata !~ m/^\s*\\#/o ) {
 		my $subclass = $class->_get_subclass($rrtype);
-		
 		return $subclass->new_from_string($self, $rdata);
 	} elsif ($RR{$rrtype}) {   # A RR type known to Net::DNS starting with \#
-		$rdata =~ m/\\\#\s+(\d+)\s+(.*)$/;
+		$rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
 
 		my $rdlength = $1;
 		my $hexdump  = $2;		
-		$hexdump =~ s/\s*//g;
+		$hexdump =~ s/\s*//og;
 
 		die "$rdata is inconsistent; length does not match content" 
 			if length($hexdump) != $rdlength*2;
@@ -350,14 +452,14 @@ sub new_from_string {
 			\$rdata, 
 			length($rdata) - $rdlength
 		);
-	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/) {   
+	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/o) {   
 		#We are now dealing with the truly unknown.
 		die 'Expected RFC3597 representation of RDATA' 
-			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/;
+			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
 
 		my $rdlength = $1;
 		my $hexdump  = $2;		
-		$hexdump =~ s/\s*//g;
+		$hexdump =~ s/\s*//og;
 
 		die "$rdata is inconsistent; length does not match content" 
 			if length($hexdump) != $rdlength*2;
@@ -383,19 +485,17 @@ sub new_from_string {
 
 sub new_from_hash {
 	my $class    = shift;
-	my %tempself = @_;
+	my %keyval   = @_;
 	my $self     = {};
-	
-	my ($key, $val);
 
-	while (($key, $val) = each %tempself) {
-		$self->{lc($key)} = $val;
+
+
+	while ( my ($key, $val) = each %keyval ) {
+	        $self->{lc $key} = $val ;
 	}
 
-	Carp::croak('RR name not specified')
-		unless exists $self->{'name'};
-	Carp::croak('RR type not specified')
-		unless exists $self->{'type'};
+	croak('RR name not specified') unless defined $self->{name};
+	croak('RR type not specified') unless defined $self->{type};
 
 	$self->{'ttl'}   ||= 0;
 	$self->{'class'} ||= 'IN';
@@ -405,18 +505,18 @@ sub new_from_hash {
 
 	if ($RR{$self->{'type'}}) {
 		my $subclass = $class->_get_subclass($self->{'type'});
-	   
 	    if (uc $self->{'type'} ne 'OPT') {
-			bless $self, $subclass;
-			
-			return $self;
+		        bless $self, $subclass;
+			$self->_normalize_dnames();
+			return _normalize_rdata($self);
+
 	    } else {  
 			# Special processing of OPT. Since TTL and CLASS are
 			# set by other variables. See Net::DNS::RR::OPT 
 			# documentation
 			return $subclass->new_from_hash($self);
 	    }
-	} elsif ($self->{'type'} =~ /TYPE\d+/) {
+	} elsif ($self->{'type'} =~ /TYPE\d+/o) {
 		bless $self, 'Net::DNS::RR::Unknown';
 		return $self;
 	} else {
@@ -425,6 +525,100 @@ sub new_from_hash {
 	}
 }
 
+
+
+# Normalizes the content of the rdata so that comparing can be done between
+# RRs created via various methods.
+
+# Based on first creating packet format and then parsing it.
+
+sub _normalize_rdata {
+	my $self     = shift;
+
+
+
+	# There are a bunch of META RR types we do not want to mess with
+	return $self if ( ( uc $self ->{'type'} eq "TSIG" )||
+			  ( uc $self ->{'type'} eq "TKEY" )
+			);
+
+
+	my $pkt = {	header		=> Net::DNS::Header->new,
+			question	=> [],
+			answer		=> [],
+			authority	=> [],
+			additional	=> []	};
+	
+	bless $pkt, "Net::DNS::Packet";
+	$pkt->push( answer => $self );
+	my $pkt2 = Net::DNS::Packet->new( \$pkt->data );
+	undef ($self);
+	return ($pkt2->answer)[0];
+}
+
+# When new_from_hash is used to generate the objects then it may be
+# that the names passed are not consistently FQDN or not.  Note that
+# the internal storage is without trailing dot.  this function
+# normalizes the domain names and is implemented in the records
+# themselves if more specific handling is needed
+
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+}
+
+
+sub _normalize_ownername {
+	my $self=shift;	
+	return $self->{'name'}=stripdot($self->{'name'});
+}
+
+
+=head2 parse
+
+    ($rrobj, $offset) = Net::DNS::RR->parse(\$data, $offset);
+
+Parses a DNS resource record at the specified location within a DNS packet.
+The first argument is a reference to the packet data.
+The second argument is the offset within the packet where the resource record begins.
+
+Returns a Net::DNS::RR object and the offset of the next location in the packet.
+
+Parsing is aborted if the object could not be created (e.g., corrupt or insufficient data).
+
+=cut
+
+use constant PACKED_LENGTH => length pack 'n2 N n', (0)x4;
+
+sub parse {
+	my ($objclass, $data, $offset) = @_;
+
+	my ($name, $index) = Net::DNS::Packet::dn_expand($data, $offset);
+	die 'Exception: corrupt or incomplete data' unless $index;
+
+	my $rdindex = $index + PACKED_LENGTH;
+	die 'Exception: incomplete data' if length $$data < $rdindex;
+	my ($type, $class, $ttl, $rdlength) = unpack("\@$index n2 N n", $$data);
+
+	my $next = $rdindex + $rdlength;
+	die 'Exception: incomplete data' if length $$data < $next;
+
+	$type = Net::DNS::typesbyval($type) || $type;
+
+	# Special case for OPT RR where CLASS should be
+	# interpreted as 16 bit unsigned (RFC2671, 4.3)
+	if ($type ne 'OPT') {
+		$class = Net::DNS::classesbyval($class) || $class;
+	}
+	# else just retain numerical value
+
+	my $self = $objclass->new_from_data($name, $type, $class, $ttl, $rdlength, $data, $rdindex);
+	die 'Exception: corrupt or incomplete RR subtype data' unless defined $self;
+
+	return wantarray ? ($self, $next) : $self;
+}
+
+
 #
 # Some people have reported that Net::DNS dies because AUTOLOAD picks up
 # calls to DESTROY.
@@ -435,36 +629,33 @@ sub DESTROY {}
 
     $rr->print;
 
-Prints the record to the standard output.  Calls the
-B<string> method to get the RR's string representation.
+Prints the record to the standard output.  Calls the B<string> method
+to get the RR's string representation.
 
 =cut
 #' someone said that emacs gets screwy here.  Who am I to claim otherwise...
 
-sub print {
-	my $self = shift;
-	print $self->string, "\n";
-}
+sub print {	print &string, "\n"; }
 
 =head2 string
 
     print $rr->string, "\n";
 
-Returns a string representation of the RR.  Calls the
-B<rdatastr> method to get the RR-specific data.
+Returns a string representation of the RR.  Calls the B<rdatastr>
+method to get the RR-specific data. Domain names arereturned in
+RFC1035 format, i.e. all non letter, digit, hyphen characters are
+represented as \DDD. Besides, all domain names are expanded to fully
+qualified domain names, with trailing dot.  This is in contrast to
+accessor methods of individual data elements in RR objects, like
+B<name>, which will not return the trailing dot.
 
 =cut
 
 sub string {
 	my $self = shift;
-
-	return join("\t",
-		"$self->{'name'}.",
-	 	 $self->{'ttl'},
-		 $self->{'class'},
-		 $self->{'type'},
-		 (defined $self->rdatastr and length $self->rdatastr) ? $self->rdatastr : '; no data',
-   );
+	my $data = $self->rdatastr || '; no data';
+  
+	join "\t", "$self->{name}.", $self->{ttl}, $self->{class}, $self->{type}, $data;
 }
 
 =head2 rdatastr
@@ -574,21 +765,23 @@ sub data {
 
 	# Don't compress TSIG or TKEY names and don't mess with EDNS0 packets
 	if (uc($self->{'type'}) eq 'TSIG' || uc($self->{'type'}) eq 'TKEY') {
-		my $tmp_packet = Net::DNS::Packet->new('');
+		my $tmp_packet = Net::DNS::Packet->new();
 		$data = $tmp_packet->dn_comp($self->{'name'}, 0);
+		return undef unless defined $data;
 	} elsif (uc($self->{'type'}) eq 'OPT') {
-		my $tmp_packet = Net::DNS::Packet->new('');
+		my $tmp_packet = Net::DNS::Packet->new();
 		$data = $tmp_packet->dn_comp('', 0);
 	} else {
 		$data  = $packet->dn_comp($self->{'name'}, $offset);
+		return undef unless defined $data;	
 	}
 
 	my $qtype     = uc($self->{'type'});
-	my $qtype_val = ($qtype =~ m/^\d+$/) ? $qtype : Net::DNS::typesbyname($qtype);
+	my $qtype_val = ($qtype =~ m/^\d+$/o) ? $qtype : Net::DNS::typesbyname($qtype);
 	$qtype_val    = 0 if !defined($qtype_val);
 
 	my $qclass     = uc($self->{'class'});
-	my $qclass_val = ($qclass =~ m/^\d+$/) ? $qclass : Net::DNS::classesbyname($qclass);
+	my $qclass_val = ($qclass =~ m/^\d+$/o) ? $qclass : Net::DNS::classesbyname($qclass);
 	$qclass_val    = 0 if !defined($qclass_val);
 	$data .= pack('n', $qtype_val);
 	
@@ -607,7 +800,7 @@ sub data {
 	my $rdata = $self->rdata($packet, $offset);
 
 	$data .= pack('n', length $rdata);
-	$data .= $rdata;
+	$data.=$rdata;
 
 	return $data;
 }
@@ -627,12 +820,13 @@ sub _canonicaldata {
 	my $self = shift;
 	my $data='';
 	{   
-		my @dname= split /\./,lc($self->{'name'});
-		for (my $i=0;$i<@dname;$i++){
-			$data .= pack ('C',length $dname[$i] );
-			$data .= $dname[$i] ;
-		}
-		$data .= pack ('C','0');
+	    my $name=$self->{'name'};
+	    my @dname=Net::DNS::name2labels($name);
+	    for (my $i=0;$i<@dname;$i++){
+		$data .= pack ('C',length $dname[$i] );
+		$data .= lc($dname[$i] );
+	    }
+	    $data .= pack ('C','0');
 	}
 	$data .= pack('n', Net::DNS::typesbyname(uc($self->{'type'})));
 	$data .= pack('n', Net::DNS::classesbyname(uc($self->{'class'})));
@@ -652,73 +846,86 @@ sub _canonicaldata {
 # have domain names in them. Verification works only on RRs with
 # uncompressed domain names. (Canonical format as in sect 8 of
 # RFC2535) _canonicalRdata is overwritten in those RR objects that
-# have domain names in the RDATA and _name2label is used to convert a
+# have domain names in the RDATA and _name2wire is used to convert a
 # domain name to "wire format"
 
+
 sub _canonicalRdata {
-    my $self = shift;
-    my $rdata = $self->rr_rdata;
+    my $self=shift;
+    my $packet=Net::DNS::Packet->new();
+    my $rdata = $self->rr_rdata($packet,0);
     return $rdata;
 }
 
 
 
-sub _name2wire   {   
-    my ($self,$name)=@_;
 
-    my $rdata = '';
-    my @dname = split(m/\./, lc $name);
-    
+
+sub _name2wire   {   
+    my ($self, $name) = @_;
+
+    my $rdata="";
+    my $compname = "";
+    my @dname = Net::DNS::name2labels($name);
+
+
     for (@dname) {
 		$rdata .= pack('C', length $_);
 		$rdata .= $_ ;
     }
     
     $rdata .= pack('C', '0');
-    
     return $rdata;
 }
 
+
+
+
+
 sub AUTOLOAD {
 	my ($self) = @_;  # If we do shift here, it will mess up the goto below.
-	
-	my ($name) = $AUTOLOAD =~ m/^.*::(.*)$/;
-	
+	my ($name) = $AUTOLOAD =~ m/^.*::(.*)$/o;
+	if ($name =~ /set_rrsort_func/){
+	    return Net::DNS::RR::set_rrsort_func(@_);
+	}
+	if ($name =~ /get_rrsort_func/){
+	    return Net::DNS::RR::get_rrsort_func(@_);
+	}
 	# XXX -- We should test that we do in fact carp on unknown methods.	
 	unless (exists $self->{$name}) {
-		my $rr_string = $self->string;
-		Carp::carp(<<"AMEN");
-
+	    my $rr_string = $self->string;
+	    Carp::carp(<<"AMEN");
+	    
 ***
 ***  WARNING!!!  The program has attempted to call the method
 ***  "$name" for the following RR object:
 ***
 ***  $rr_string
 ***
-***  This object doesn't have a method "$name".  THIS IS A BUG
+***  This object does not have a method "$name".  THIS IS A BUG
 ***  IN THE CALLING SOFTWARE, which has incorrectly assumed that
 ***  the object would be of a particular type.  The calling
 ***  software should check the type of each RR object before
 ***  calling any of its methods.
 ***
 ***  Net::DNS has returned undef to the caller.
-***
+*** 
 
 AMEN
-		return;
+return;
 	}
 	
 	no strict q/refs/;
 	
 	# Build a method in the class.
 	*{$AUTOLOAD} = sub {
-		my ($self, $new_val) = @_;
-				
-		if (defined $new_val) {
-			$self->{$name} = $new_val;
-		}
-		
-		return $self->{$name};
+	    my ($self, $new_val) = @_;
+	    
+	    if (defined $new_val) {
+		$self->{$name} = $new_val;
+	    }
+	    
+	    return $self->{$name};
 	};
 	
 	# And jump over to it.
@@ -726,6 +933,7 @@ AMEN
 }
 
 
+
 #
 #  Net::DNS::RR->_get_subclass($type)
 #
@@ -746,7 +954,115 @@ sub _get_subclass {
 	
 	return $subclass;
 }	
+
+
+
+
+=head1 Sorting of RR arrays
+
+As of version 0.55 there is functionality to help you sort RR
+arrays. The sorting is done by Net::DNS::rrsort(), see the
+L<Net::DNS> documentation. This package provides class methods to set
+the sorting functions used for a particular RR based on a particular
+attribute.
+
+
+=head2 set_rrsort_func
+
+Net::DNS::RR::SRV->set_rrsort_func("priority",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->priority <=> $b->priority
+				   ||
+				   $b->weight <=> $a->weight
+                     }
+
+Net::DNS::RR::SRV->set_rrsort_func("default_sort",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->priority <=> $b->priority
+				   ||
+				   $b->weight <=> $a->weight
+                     }
+
+set_rrsort_func needs to be called as a class method. The first
+argument is the attribute name on which the sorting will need to take
+place. If you specify "default_sort" than that is the sort algorithm
+that will be used in the case that rrsort() is called without an RR
+attribute as argument.
+
+The second argument is a reference to a function that uses the
+variables $a and $b global to the C<from Net::DNS>(!!)package for the
+sorting. During the sorting $a and $b will contain references to
+objects from the class you called the set_prop_sort from. In other
+words, you can rest assured that the above sorting function will only
+get Net::DNS::RR::SRV objects.
+
+The above example is the sorting function that actually is implemented in 
+SRV.
+
+=cut
+
+
+
+
+sub set_rrsort_func{
+    my $class=shift;
+    my $attribute=shift;
+    my $funct=shift;
+#    print "Using ".__PACKAGE__."set_rrsort: $class\n";
+    my ($type) = $class =~ m/^.*::(.*)$/o;
+    $Net::DNS::RR::rrsortfunct{$type}{$attribute}=$funct;
+}
 		
+sub get_rrsort_func {
+    my $class=shift;    
+    my $attribute=shift;  #can be undefined.
+    my $sortsub;
+    my ($type) = $class =~ m/^.*::(.*)$/o;
+
+
+#    print "Using ".__PACKAGE__." get_rrsort: $class ($type,$attribute)\n";
+#    use Data::Dumper;
+#    print Dumper %Net::DNS::rrsortfunct;
+
+    if (defined($attribute) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}{$attribute})
+	){
+	#  The default overwritten by the class variable in Net::DNS
+	return $Net::DNS::RR::rrsortfunct{$type}{$attribute};
+    }elsif(
+	! defined($attribute) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}) &&
+	exists($Net::DNS::RR::rrsortfunct{$type}{'default_sort'})
+	){
+	#  The default overwritten by the class variable in Net::DNS
+	return $Net::DNS::RR::rrsortfunct{$type}{'default_sort'};
+    }
+    elsif( defined($attribute) ){
+	
+	return sub{
+	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+	    ( exists($a->{$attribute}) &&   
+	      $a->{$attribute} <=> $b->{$attribute})
+		||
+		$a->_canonicaldata() cmp $b->_canonicaldata()
+	};
+    }else{
+	return sub{
+	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+	    $a->_canonicaldata() cmp $b->_canonicaldata()
+	};
+    }    
+
+    return $sortsub;
+}
+
+
+
+
+
 	
 sub STORABLE_freeze {
 	my ($self, $cloning) = @_;
@@ -777,6 +1093,10 @@ Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
+Portions Copyright (c) 2005-2007 Olaf Kolkman 
+
+Portions Copyright (c) 2007 Dick Franks 
+
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
 
