@@ -1,12 +1,15 @@
 package Net::DNS::RR::MINFO;
 #
-# $Id: MINFO.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: MINFO.pm 718 2008-02-26 21:49:20Z olaf $
 #
 use strict;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+$VERSION = (qw$LastChangedRevision: 718 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -23,10 +26,8 @@ sub new_from_string {
 	my ($class, $self, $string) = @_;
 
 	if ($string && ($string =~ /^(\S+)\s+(\S+)$/)) {
-		$self->{"rmailbx"} = $1;
-		$self->{"emailbx"} = $2;
-		$self->{"rmailbx"} =~ s/\.+$//;
-		$self->{"emailbx"} =~ s/\.+$//;
+		$self->{"rmailbx"} = Net::DNS::stripdot($1);
+		$self->{"emailbx"} = Net::DNS::stripdot($2);
 	}
 
 	return bless $self, $class;
@@ -55,13 +56,22 @@ sub rr_rdata {
 }
 
 
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+	$self->{'rmailbx'}=Net::DNS::stripdot($self->{'rmailbx'}) if defined $self->{'rmailbx'};
+	$self->{'emailbx'}=Net::DNS::stripdot($self->{'emailbx'}) if defined $self->{'emailbx'};
+}
+
+
+
 sub _canonicalRdata {
 	my ($self, $packet, $offset) = @_;
 	my $rdata = "";
 
 	if (exists $self->{"rmailbx"}) {
-		$rdata .= $self->_name2wire($self->{"rmailbx"});
-		$rdata .=  $self->_name2wire($self->{"emailbx"});
+		$rdata .= $self->_name2wire(lc($self->{"rmailbx"}));
+		$rdata .=  $self->_name2wire(lc($self->{"emailbx"}));
 	}
 
 	return $rdata;
