@@ -1,12 +1,15 @@
 package Net::DNS::RR::PTR;
 #
-# $Id: PTR.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: PTR.pm 812 2009-11-26 15:04:14Z olaf $
 #
 use strict;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+$VERSION = (qw$LastChangedRevision: 812 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -22,8 +25,7 @@ sub new_from_string {
 	my ($class, $self, $string) = @_;
 
 	if ($string) {
-		$string =~ s/\.+$//;
-		$self->{"ptrdname"} = $string;
+		$self->{"ptrdname"} = Net::DNS::stripdot($string);
 	}
 
 	return bless $self, $class;
@@ -31,8 +33,8 @@ sub new_from_string {
 
 sub rdatastr {
 	my $self = shift;
-
-	return $self->{"ptrdname"} ? "$self->{ptrdname}." : '';
+	return "" unless defined $self->{ptrdname};
+	return "$self->{ptrdname}." ;
 }
 
 sub rr_rdata {
@@ -40,12 +42,21 @@ sub rr_rdata {
 	my $rdata = "";
 
 	if (exists $self->{"ptrdname"}) {
-		$rdata .= $packet->dn_comp($self->{"ptrdname"}, $offset);
+		$rdata .= $packet->dn_comp(lc($self->{"ptrdname"}), $offset);
 	}
 
 	return $rdata;
 }
 
+
+
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+	$self->{'ptrdname'}=Net::DNS::stripdot($self->{'ptrdname'}) if defined $self->{'ptrdname'};
+}
+
+
 sub  _canonicalRdata {
 	my ($self, $packet, $offset) = @_;
 	my $rdata = "";
