@@ -1,12 +1,43 @@
 package Net::DNS::RR::SRV;
 #
-# $Id: SRV.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: SRV.pm 718 2008-02-26 21:49:20Z olaf $
 #
 use strict;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+$VERSION = (qw$LastChangedRevision: 718 $)[1];
+
+
+
+__PACKAGE__->set_rrsort_func("priority",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->{'priority'} <=> $b->{'priority'}
+				   ||
+				       $b->{'weight'} <=> $a->{'weight'}
+}
+);
+
+
+__PACKAGE__->set_rrsort_func("default_sort",
+			       __PACKAGE__->get_rrsort_func("priority")
+
+    );
+
+__PACKAGE__->set_rrsort_func("weight",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $b->{"weight"} <=> $a->{"weight"}
+				   ||
+				       $a->{"priority"} <=> $b->{"priority"}
+}
+);
+
+
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -39,6 +70,7 @@ sub rdatastr {
 
 	if (exists $self->{'priority'}) {
 		$rdatastr = join(' ', @{$self}{qw(priority weight port target)});
+		$rdatastr =~ s/(.*[^\.])$/$1./;
 	} else {
 		$rdatastr = '';
 	}
@@ -52,25 +84,23 @@ sub rr_rdata {
 
 	if (exists $self->{'priority'}) {
 		$rdata .= pack('n3', @{$self}{qw(priority weight port)});
-		$rdata .= $packet->dn_comp($self->{'target'}, $offset + length $rdata);
-	}
+		$rdata .=  $self->_name2wire ($self->{"target"});
 
-	return $rdata;
-}
-
-
-sub _canonicalRdata {
-	my $self  = shift;
-	my $rdata = '';
+	      }
 	
-	if (exists $self->{'priority'}) {
-		$rdata .= pack('n3', @{$self}{qw(priority weight port)});
-		$rdata .= $self->name_2wire($self->{'target'});
-	}
-
 	return $rdata;
 }
 
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+	$self->{'target'}=Net::DNS::stripdot($self->{'target'}) if defined $self->{'target'};
+}
+
+
+
+
+
 1;
 __END__
 
